c baryc_outlier.f
c Transform velocities to a barycentric frame, with outliers removed.
c Miroslav Broz (miroslav.broz@email.cz), Apr 15th 2015

      program baryc_outlier

      implicit none
      integer nmax
      parameter(nmax = 200000)
      real*8 pi
      parameter(pi = 4.d0*atan(1.d0))

      integer id(nmax)
      real*8 x(nmax), y(nmax), z(nmax), vx(nmax), vy(nmax), vz(nmax),
     :  h(nmax), m(nmax)
      real*8 vmax, mmin1, mmin2, rho0

c temporary variables
      integer i, j, n, ierr, k, l
      real*8 dummy, bx, by, bz, bvx, bvy, bvz, sm, v, rho, h0
      character*255 str

c read command line
      if (iargc().ne.4) then
        write(*,*) 'Usage: baryc_outlier vmax mmin1 mmin2 rho0'
        stop
      endif
      call getarg(1,str)
      read(str,*,err=990,end=990) vmax
      call getarg(2,str)
      read(str,*,err=990,end=990) mmin1
      call getarg(3,str)
      read(str,*,err=990,end=990) mmin2
      call getarg(4,str)
      read(str,*,err=990,end=990) rho0

      write(*,*) '# vmax = ', vmax, ' m/s'
      write(*,*) '# mmin1 = ', mmin1, ' kg'
      write(*,*) '# mmin2 = ', mmin2, ' kg'
      write(*,*) '# rho0 = ', rho0, ' kg/m^3'

c read standard input
      i = 0
      ierr = 0
      do while (ierr.eq.0)
        read(*, 10, iostat=ierr) str
10      format(a)
        if ((ierr.eq.0).and.(str(1:1).ne.'#')) then
          i = i+1
          if (i.gt.nmax) then
             write(*,*) 'Error: number of data .gt. nmax = ', nmax
             stop
          endif
          read(str, *, iostat=ierr) id(i), x(i), y(i), z(i),
     :      vx(i), vy(i), vz(i), h(i), m(i)
        endif
      enddo
      n = i

c compute barycenter
      bx = 0.d0
      by = 0.d0
      bz = 0.d0
      bvx = 0.d0
      bvy = 0.d0
      bvz = 0.d0
      sm = 0.d0
      j = 0
      do i = 1, n
        v = sqrt(vx(i)**2 + vy(i)**2 + vz(i)**2)
        if ((v.lt.vmax).and.(m(i).gt.mmin1)) then
          j = j + 1
          bx = bx + x(i)*m(i)
          by = by + y(i)*m(i)
          bz = bz + z(i)*m(i)
          bvx = bvx + vx(i)*m(i)
          bvy = bvy + vy(i)*m(i)
          bvz = bvz + vz(i)*m(i)
          sm = sm + m(i)
        endif
      enddo
      bx = bx/sm
      by = by/sm
      bz = bz/sm
      bvx = bvx/sm
      bvy = bvy/sm
      bvz = bvz/sm

c shift to c.o.m.
c write output
      write(*,*) '# bx = ', bx, ' m'
      write(*,*) '# by = ', by, ' m'
      write(*,*) '# bz = ', bz, ' m'
      write(*,*) '# bvx = ', bvx, ' m'
      write(*,*) '# bvy = ', bvy, ' m'
      write(*,*) '# bvz = ', bvz, ' m'
      write(*,*) '# nparticles = ', j
      write(*,*) '# \vec r shifted to c.o.m.'
      write(*,*) '# \vec v shifted to c.o.m.'
      write(*,*) '# id  x   y   z   vx    vy    vz    h   m   ',
     :  ' rho      rho0     h0'
      write(*,*) '# [1] [m] [m] [m] [m/s] [m/s] [m/s] [m] [kg]',
     :  ' [kg/m^3] [kg/m^3] [m]'

      do i = 1, n
        if (m(i).gt.mmin2) then
          x(i) = x(i) - bx
          y(i) = y(i) - by
          z(i) = z(i) - bz
          vx(i) = vx(i) - bvx
          vy(i) = vy(i) - bvy
          vz(i) = vz(i) - bvz
          v = sqrt(vx(i)**2 + vy(i)**2 + vz(i)**2)
          rho = m(i)/(4.d0/3.d0*pi*h(i)**3)
          h0 = (m(i)/(4.d0/3.d0*pi*rho0))**(1.d0/3.d0)
        
          write(*,*) id(i), x(i), y(i), z(i), vx(i), vy(i), vz(i),
     :      h(i), m(i), rho, rho0, h0
        endif
      enddo

      stop

990   continue
      write(*,*) 'Error reading command-line parameters.'
      stop

      end
      

